######
# Cochran-Armitage linear trend test
######
# based on documentation:
# http://linkage.rockefeller.edu/pawe3d/help/Linear-trend-test-ncp.html
#
# Reference:
# Gordon D, Haynes C, Blumenfeld J, Finch SJ (2005) 
#   PAWE-3D: visualizing Power for Association With Error in case/control 
#   genetic studies of complex traits. 
# Bioinformatics 21:3935-3937.
#
# Gordon D, Finch SJ, Nothnagel M, Ott J (2002) 
#   Power and sample size calculations for case-control genetic association 
#   tests when errors are present: application to single nucleotide 
#   polymorphisms. 
# Hum Hered 54:22-33.

# Cochran, W.G. (1954) 
#   Some methods for strengthening the common chi-squared tests. 
# Biometrics. 10, 417-451.
#
# Armitage, P. (1955) 
#   Tests for linear trends in proportions and frequencies. 
# Biometrics. 11, 375-386.
#
# Chapman, D.G. and Nam, J.M. (1968) 
#   Asymptotic power of chi square tests for linear trends in proportions. 
# Biometrics. 24, 315-327.
#

# INPUTS >>>>>>>>
# x -- a vector of bialleleic markers coded by additive model.
#      Denote rare allele as 'A' and common allele as 'B'.
#      x=0 -- BB; x=1 -- AB; x=2 -- AA (additive model)
#      x=0 -- BB; x=0 -- AB; x=1 -- AA (recessive model)
#      x=0 -- BB; x=1 -- AB; x=1 --AA ( dominant model)
# mem -- disease membership. 1 -- case; 0 -- control
#
# OUTPUTS >>>>>>>>>
#   test statistic and p-value of the Cochran-Armitage trend test

# Cochran-Armitage linear trend test
ArmitageTest<-function(x,mem)
{
  x<-x[!is.na(mem)]
  mem<-mem[!is.na(mem)]
  mem.u<-sort(unique(mem))
  if(length(mem.u)!=2)
  { warning("The number of unique disease status should be 2!\n") 
    res<-c(NA, NA)
    names(res)<-c("stat", "pvalue")
    return(res)
  }
  if(mem.u[1]!=0 || mem.u[2]!=1)
  { warning("The coding of 'mem' should be 1 -- case; 0 -- control!\n") 
    res<-c(NA, NA)
    names(res)<-c("stat", "pvalue")
    return(res)
  }

  mem<-mem[!is.na(x)]
  x<-x[!is.na(x)]
  if(length(unique(x))<2)
  {
    warning("There is only one genotype!\n")
    res<-c(NA, NA)
    names(res)<-c("stat", "pvalue")
    return(res)
  }

  # 'N' -- number of subjects genotyped
  N<-length(x)
  if(N<2)
  { warning("Too small number of subjects!\n") 
    res<-c(NA, NA)
    names(res)<-c("stat", "pvalue")
    return(res)
  }

  x.u<-unique(x)
  n.x.u<-length(x.u)
  # number of cases
  R<-sum(mem)
  # number of cases
  S<-N-R
  Q0<-S/N
  Q1<-R/N

   ri<-rep(0, n.x.u)
   si<-rep(0, n.x.u)
   for(i in 1:n.x.u)
   { ri[i]<-sum(mem==1 & x==x.u[i])
     si[i]<-sum(mem==0 & x==x.u[i])
   }
   ni<-ri+si

   U<-sum(x.u*(Q0*ri-Q1*si))
   # estimate of var(U)
   V=R*S*(N*sum(x.u^2*ni)-(sum(x.u*ni))^2)/(N^3)

   # test statistic
   ZT<-U/sqrt(V)

   # test statistics approximately follows a standard normal distribution 
   # under the null hypothesis that genotype frequencies for each category 
   # i are equal in cases and controls.
   # i.e., under H0, ZT~N(0, 1)

   # ZT^2 is the linear trend test statistic
   # under H0: stat~chi^2 with df=1
   stat<-ZT^2
   # pvalue
   pvalue<-1-pchisq(stat, df=1)

  res<-c(stat, pvalue)
  names(res)<-c("stat", "pvalue")
  return(res)

}

# coding genotypes
# method
# x=0 -- BB; x=1 -- AB; x=2 -- AA (additive model: A)
# x=0 -- BB; x=0 -- AB; x=1 -- AA (recessive model: R)
# x=0 -- BB; x=1 -- AB; x=1 --AA ( dominant model: D)
genotypeCoding<-function(geneSetObj, method="A")
{
  pedObj<-geneSet2Ped(geneSetObj)
  codedGenoInfo<-genotypeCoding.default(pedObj, method=method)

  return(codedGenoInfo)
}


# coding genotypes
# method
# x=0 -- BB; x=1 -- AB; x=2 -- AA (additive model: A)
# x=0 -- BB; x=0 -- AB; x=1 -- AA (recessive model: R)
# x=0 -- BB; x=1 -- AB; x=1 --AA ( dominant model: D)
genotypeCoding.default<-function(pedObj, method="A")
{
  method<-match.arg(method, choices=c("A", "R", "D"))
  columns<-pedObj$columns
  len<-length(columns)
  markerNames<-pedObj$markerNames
  nMarkers<-length(markerNames)
  genoInfo<-pedObj$ped[,-c(1:len),drop=F] 
  sampleInfo<-pedObj$ped[,c(1:len)]

  nr<-nrow(genoInfo)
  codedGenoInfo<-matrix(0, nrow=nr, ncol=nMarkers)

  for(i in 1:nMarkers)
  {
    start<-2*i-1
    end<-2*i

    # get allele info for i-th marker
    markeri<-genoInfo[,start:end]
    # stack the two columns of i-th marker
    tmpVec<-c(markeri[,1], markeri[,2])
    # remove missing alleles
    tmpVec2<-tmpVec[tmpVec!=0]
    # get minor allele and major allele
    tt<-as.numeric(names(sort(table(tmpVec2))))
    minor<-tt[1]
    major<-tt[2]
    tmpVec3<-tmpVec
    tmpVec3[tmpVec==0]<-NA 
    tmpVec3[tmpVec==minor]<-1
    tmpVec3[tmpVec==major]<-0
    markeri2<-matrix(tmpVec3, ncol=2, byrow=FALSE)
    # additive model
    if(method=="A")
    { 
      codedGenoInfo[,i]<-apply(markeri2, 1, sum) 
    } else if(method=="R"){ # recessive model
      codedGenoInfo[,i]<-apply(markeri2, 1, prod)
    } else { # dominant model
      codedGenoInfo[,i]<-apply(markeri2, 1, function(x) {return(as.numeric(x[1]|x[2]))} )
    }
  }
  colnames(codedGenoInfo)<-markerNames
  rownames(codedGenoInfo)<-rownames(sampleInfo)

  return(codedGenoInfo)
}

Armitage<-function(geneSetObj, method="A")
{
  pedObj<-geneSet2Ped(geneSetObj)
  res<-Armitage.default(pedObj, method=method)
  return(res)
}

Armitage.default<-function(pedObj, method="A")
{
  # get sample info
  columns<-pedObj$columns
  len<-length(columns)
  if(len!=6)
  { stop("No affection status in the 'pedObj'!") }
  markerNames<-pedObj$markerNames
  nMarkers<-length(markerNames)
  sampleInfo<-pedObj$ped[,c(1:6)]

  # get affection status
  mem<-as.numeric(sampleInfo[,6])
  if(length(unique(mem[mem!=0]))!=2)
  { stop("The number of unique affection status is not equal to 2!") }

  # 0 -- unknnown; 1 -- unaffected; 2 -- affected
  mem2<-mem
  mem2[mem==0]<-NA
  mem2[mem==1]<-0
  mem2[mem==2]<-1
  mem<-mem2

  # get genotype coding
  codedGenoInfo<-genotypeCoding.default(pedObj, method=method)


  res<-t(apply(codedGenoInfo, 2, ArmitageTest, mem=mem))
  colnames(res)<-c("stat", "pvalue")
  rownames(res)<-markerNames

  return(res)
}

