##
# Visualizing r^2 or D' matrix
##

###########
# LdMat -- (lower triangle matrix of) pair-wise r^2 or D'
# SNPloc -- SNP locations
# is.triangle - indicate if 'LdMat' is a lower triangle matrix or not
#               if 'LdMat' is an upper triangle, then the user has to
#               transpose it before calling the function 'LDView'.
# SNPnames -- labels for SNPs
# cexSNPnames -- font size for SNPs labels
# margins -- margins for heatmap
# main -- title for the plot 
# barCol -- specify the color scheme
# widths -- The plot is split into two parts -- upper part and lower part
#           'widths' specifies the widths of the two parts
# heights -- The plot is split into two parts -- upper part and lower part
#           'heights' specifies the heights of the two parts
# ... -- other parameters for the function 'image'
###########

LDView<-function(LdMat, 
                 SNPloc=NULL,
                 is.triangle=TRUE, 
                 SNPnames=NULL, 
                 cexSNPnames=0.2+1/log10(nrow(LdMat)), 
                 margins=c(6,2,3,6), 
                 main=NULL, 
                 barCol=rainbow(256,start=0,end=1/6 ),
                 widths=c(10,10), 
                 heights=c(1,10),
                 ...)
{
  n<-nrow(LdMat)
  theta<-pi/4
  # sort SNPs in accending order of their locations.
  if(!is.null(SNPloc))
  { posMat<-cbind(1:n, SNPloc)
    posMat2<-posMat[order(posMat[,2]),1:2]

    LdMat2<-LdMat[posMat2[,1], posMat2[,1]]
    SNPloc2<-SNPloc[posMat2[,1]]
    if(length(SNPloc2)!=n)
    { stop("Length of 'SNPloc' does not match the number of SNPs!\n") }
    SNPnames2<-SNPnames[posMat2[,1]]
  } else {
    LdMat2<-LdMat
    SNPnames2<-SNPnames
    SNPloc2<-SNPloc
  }
  rownames(LdMat2)<-SNPnames2
  colnames(LdMat2)<-SNPnames2

  # if 'LdMat' is a square matrix, transform it to lower triangle matrix.
  # if 'LdMat' is a upper triangle matrix, then the user has to transpose
  # it to a lower triangle matrix.
  if(is.triangle==FALSE)
  {
    for(i in 1:(n-1))
    { for(j in (i+1):n)
      { LdMat2[i,j]<-NA }
    }
  }

  # divide the device into two rows and one column
  # allocate figure 1 all of row 1
  # allocate figure 2 all of row 2
  layout(matrix(c(1,2),2,1,byrow=T), widths=widths, heights=heights,respect = TRUE)
  n.color<-length(barCol)
  # figure 1 -- a color bar showing color scheme
  par(mar = c(1, 1, 1, 1))
  image(cbind(1:n.color), col = barCol, axes = FALSE)
  axis(1, c(0,0.5, 1), labels=c("0","0.5","1"), tick=0)

  # title of the plot
  if(!is.null(main))
  { title(main) }

  # figure 2
  par(mar = margins)
  image(x=1:n, y=1:n, z=LdMat2, col=barCol, axes=FALSE, xlab="", ylab="",...)
  axis(1, 1:n, labels=SNPnames2, las=2, line=-0.5, tick=0, cex.axis=cexSNPnames)
  axis(4, 1:n, labels=SNPnames2, las=2, line=-0.5, tick=0, cex.axis=cexSNPnames)

  if(is.null(SNPloc)==FALSE) # draw lines to indicate locations of SNPs
  { n.min<-1/2
    n.max<-n+1/2
    n.avg<-(n.min+n.max)/2
    lines(x=c(-n.avg+1/2,n.max), y=c(0,n+n.avg), lwd=2)
 
    # draw lines to indicate locations of SNPs
    # for each SNP, scale its position from [min(SNPloc2), max(SNPloc2)] 
    # to [1/2, ell+1/2] where 'ell' is the length of line segment used
    # to indicate the locations of SNPs
    tmpx<-tapply(SNPloc2, 1:n, scaling, low=min(SNPloc2), upp=max(SNPloc2), n=n)
    # coordinates of  points
    tmpmat<-cbind(tmpx, rep(n.avg,n))
 
    # rotation and scaling 
    tmpmat2<-t(apply(tmpmat, 1, rotation, n.avg=n.avg, theta=theta))
    # drawing points to indicate SNP locations
    points(tmpmat2[,1], tmpmat2[,2], col=2, pch=4, cex=1, lwd=2)
 
    # points on y=x
    tmpmat3<-cbind(1:n, 1:n)

    tmpmat4<-cbind(tmpmat2, tmpmat3)
    apply(tmpmat4, 1, mylines)
 
    axis(2, n.avg, labels=round(min(SNPloc2)), las=2, line=-0.5, tick=0, cex.axis=1)
    axis(3, n.avg, labels=round(max(SNPloc2)), las=1, line=-0.5, tick=0, cex.axis=1)
 
  }
  par(mar = c(0, 0, if (!is.null(main)) 1 else 0, margins[2]))

  invisible(list(SNPnames=SNPnames2, LdMat=LdMat2, SNPloc=SNPloc2))
}


# transform a value between [low, upp] to a value between [0, n]
scaling<-function(x, low, upp, n)
{
  if(x<low || x>upp)
  { msg<-paste(x, "should be in interval [", low, ",", upp, "]\n")
    stop(msg)
  }
  n.max<-n+1/2
  n.min<-1/2
  n.avg<-(n.min+n.max)/2

  x0<-1/2
  y0<-n.avg
  x1<-n.avg
  y1<-n.max
  ell<-sqrt((x1-x0)^2+(y1-y0)^2)
  ell1<-upp-low

  xnew<-ell*(x-low)/ell1+1/2

  return(xnew)
}

# rotate horizontal line with 'theta' angle
rotation<-function(vec, n.avg, theta=pi/4)
{
  x<-vec[1]
  y<-vec[2]
  x2<-(x-0.5)*cos(theta)+1/2
  y2<-(x-0.5)*sin(theta)+n.avg
  return(c(x2, y2))
}

mylines<-function(vec)
{
  x1<-vec[1]
  y1<-vec[2]
  x2<-vec[3]
  y2<-vec[4]

  lines(x=c(x1, x2), y=c(y1, y2), lwd=1, col=1)
}

