# $Id: geneSet-class.R 1110 2006-11-21 22:13:37Z warnes $

setClass("geneSet",
	 representation(
		callCodes = "matrix",
		errorMetrics = "matrix",
		transTables = "list",
		missingCodes = "list",
		sampleInfo = "data.frame", 
		markerInfo = "data.frame", 
		studyInfo = "list", 
		description = "character",
		notes = "character",
		ploidy = "numeric",
		phase = "list"), 
	prototype = list(
		callCodes = matrix(as.integer(0), nc = 0, nr = 0), 
		errorMetrics = matrix(0., nc = 0, nr = 0), 
		transTables = list(), 
		missingCodes = list(), 
		sampleInfo = data.frame(), 
		markerInfo = data.frame(), 
		studyInfo = list(),
		description = "",
		notes = "",
		ploidy = numeric(0),
		phase = list()
	)) 


## Check validity of geneSet object
validate <- function(object, quiet=TRUE)
  {
    if (!quiet) cat("Validating...")
    
    callCodes    = object@callCodes
    errorMetrics = object@errorMetrics
    transTables  = object@transTables 
    missingCodes = object@missingCodes
    sampleInfo   = object@sampleInfo
    markerInfo   = object@markerInfo
    studyInfo    = object@studyInfo
    description  = object@description
    notes        = object@notes
    ploidy       = object@ploidy
    phase        = object@phase

    ## Check nrow(smapleInfo) == ncol(callCodes)
    if ( nrow(sampleInfo) != ncol(callCodes) )
      stop("Number of rows of sampleInfo differs from the ",
           "number of columns of callCodes ") 

    ## Check thet markerInfo has the necessary columns
    required.names <- c("Name","TransTable")
    ## if( !all( required.names %in% colnames(markerInfo) ) )
    ##   stop("markerInfo must contain columns:", required.names )
        
    ## Ensure rownames(markerInfo) == markerInfo$Name
    rownames(markerInfo) <- markerInfo$Name

    ## Ensure rownames(callCodes) == markerInfo$Name 
    rownames(callCodes) <- markerInfo$Name
    
    ## Check transTable Names are in markerInfo$transTable
    if ( (!all(names(transTables) %in% markerInfo$TransTable)) ||
         (!all(markerInfo$TransTable %in% names(transTables))) )
      stop("Names of transTables and markerInfo$TransTable don't match")

    ## Check missingCodes Names are in markerInfo$transTable
    ## if ( (!all(names(missingCodes) %in% markerInfo$TransTable)) ||
    ##     (!all(markerInfo$TransTable %in% names(missingCodes))) )
    ##  stop("Names of missingCodes and markerInfo$TransTable don't match")
    
    ## Check if phase is TRUE/FASLE
    if ( !all(unlist(phase) %in% c(TRUE, FALSE)) )
      stop("phase must contain only TRUE and FALSE")

    ## Check if ploidy is an integer
    if (ploidy !=as.integer(ploidy) )
      stop("Ploidy must be an integer")
    
    ## check that all referenced transTables are present
    if( !all( markerInfo$TransTable %in% names(transTables) ) )
      stop("All referenced transTables are NOT present in markerInfo$TransTable")


    ## check that every value in callCodes is present
    ## in the corresponding transTable
    markerList <- split(markerInfo$Name, markerInfo$TransTable)

    checkCodes <- function(tName, mNames)
      {
        ## since we code data 1:numUniqueGenotypes, we only need to
        ## check if the max(callCodes) is greater than
        ## numUniqueGenotypes
        max( callCodes[mNames,], na.rm=TRUE) <= nrow( transTables[[tName]] )
      }   

   for(tName in names(markerList))
      {
        if(!quiet) cat("Validating transTable", tName, ".." )
        mNames <- markerList[[tName]]

        flag <- checkCodes(tName, mNames)

        # find the specific column with issues
        if(!flag)
          for(marker in mNames)
            {
              flag <- checkCodes(tName, marker)
              if(!flag)
                stop("callCodes contains value(s) not present in ",
                     "the transTable for marker '", marker, "'")
            }
        if(!quiet) cat("Done.\n")
      }

    if(!quiet) cat("Done\n")
    object
  }

## Constructor ##
geneSet <- function(callCodes,
                    errorMetrics=matrix(nrow=0,ncol=0),
                    transTables, 
                    missingCodes,
                    sampleInfo,
                    markerInfo,
                    studyInfo=list(),
                    description="",
                    notes="",
                    ploidy=2,
                    phase=list(FALSE)
                    )
  {

    if(missing(missingCodes))
      {
        missingCodes <- sapply( transTables, function(...) c() )
      }
    
    ## build it
    retval <- new("geneSet",
                  callCodes    = as.matrix(callCodes),
                  errorMetrics = as.matrix(errorMetrics),
                  transTables  = as.list(transTables),
                  missingCodes = as.list(missingCodes),
                  sampleInfo   = as.data.frame(sampleInfo),
                  markerInfo   = as.data.frame(markerInfo),
                  studyInfo    = as.list(studyInfo) ,
                  description  = as.character(description),
                  notes        = as.character(notes),
                  ploidy       = as.integer(ploidy),
                  phase        = as.list(phase)
                  )

    ## check it
    validate(retval)

    ## return it
    retval
  }


# Check for a geneSet object
is.geneSet <- function(x)
  inherits(x, "geneSet")

### constructor
#setMethod("initialize", "geneSet", 
#	  function(.Object, 
#                   geneData,  
#                   sampleData, 
#                   format=c('separator',  'columns',  'count',  'onestring'),
#                   sep='/',  # character separator, or character index between components
#                   reorder=c('frequency', 'specified', FALSE, 'alphabetic') ,
#                   alleles,
#                   missing.value=NA,
#                   ploidy=2)
#	  {
#	    
#
#	  }
#	  )

