//////////////////////////////////////////////////////////////////////
// LdMax.cpp
// (c) 2000-2001 Goncalo Abecasis (goncalo@umich.edu)
//
// This file was derived with kind permission from the original author (Dr Abecasis),
// from material distributed as part of the GOLD (Bioinformatics 16:182-3, 2000)
// source code package which is available in it's original form from
// http://www.sph.umich.edu/csg/abecasis/GOLD/
//
// If this library proves useful to you, please ensure the original source is properly
// attributed using the citation above.
//
// This library is free software; you can redistribute it and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software Foundation; either version
// 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
// even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License along with this library;
// if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
// 02111-1307 USA or try http://www.opensource.org/licenses/lgpl-license.php
//
// Please contact the project maintainers responsible for the distribution of this library for
// technical support, not the author.
//
// To request permission to use this source code library other than under the LGPL, please
// contact the copyright holder, Dr Abecasis directly at goncalo@umich.edu
// Original header follows:
////////////////////////////////////////////////////////////////////////
// ldmax/LdMax.h
// (c) 2000-2001 Goncalo Abecasis
//
// This file is distributed as part of the GOLD source code package
// and may not be redistributed in any form, without prior written
// permission from the author. Permission is granted for you to
// modify this file for your own personal use, but modified versions
// must retain this copyright notice and must not be distributed.
//
// Permission is granted for you to use this file to compile GOLD.
//
// All computer programs have bugs. Use this file at your own risk.
//
// Thursday November 08, 2001
//
/**************
*
* GOLD source code reference
*
* @article{AbecasisCookson:2000,
*   author="{Abecasis, G. R.} and {Cookson, W. O. C.}",
*   title={{GOLD} --- Graphical Overview of Linkage Disequilibrium},
*   journal={Bioinformatics},
*   volume={16},
*   pages={182-183},
*   year={2000}
* }
*
*
***************/

/****************
 Dr. Albert Vernon Smith made a great contribution to improve
 this file.

Albert Vernon Smith, Ph.D.
Cold Spring Harbor Laboratory
HapMap Data Coordination Center
E-mail: smithav@cshl.edu
****************/

#include "LdMax.h"



LDMax::LDMax()
   {
   NullWorkingArrays();

   defaults = (double *) NULL;
   estimates = (double *) NULL;

   ngeno = nhaplo = nloci = 0;
   foundersOnly = true;
   }

LDMax::~LDMax()
   {
   FreeWorkingArrays();

   if (defaults)
      delete [] defaults;
   if (estimates)
      delete [] estimates;
   }

void LDMax::NullWorkingArrays()
   {
   alleleFrequencies = (double **) NULL;
   haplotypeFrequencies = (double *) NULL;
   genotypeFrequencies = (double *) NULL;

   haplotypeList1 = (int **) NULL;
   haplotypeList2 = (int **) NULL;

   haplotype1 = (int *) NULL;
   haplotype2 = (int *) NULL;
   genotype = (int *) NULL;
   genotypeCounts = (int *) NULL;

   markers = (int *) NULL;
   nAlleles = (int *) NULL;
   nGenotypes = (int *) NULL;

   lenHaploLists = (int *) NULL;

  }

void LDMax::FreeWorkingArrays()
   {
   if (alleleFrequencies != NULL)
      {
        for (int i = 0; i < nloci; i++)
        { delete[] alleleFrequencies[i]; }
	// free(alleleFrequencies); Automatically peformed for R_alloc
      }
   if (haplotypeFrequencies)
      delete [] haplotypeFrequencies;

   if (genotypeFrequencies)
      delete [] genotypeFrequencies;

   if (haplotypeList1 != NULL) 
      {
        for (int i = 0; i < ngeno; i++)
        { delete[] haplotypeList1[i]; }
        // free(haplotypeList1); Automatically peformed for R_alloc
      }
   if (haplotypeList2 != NULL) 
      {
        for (int i = 0; i < ngeno; i++)
        { delete[] haplotypeList2[i]; }
        // free(haplotypeList2); Automatically peformed for R_alloc
      }
   if (lenHaploLists)
      delete [] lenHaploLists;

   if (haplotype1)
      delete [] haplotype1;
   if (haplotype2)
      delete [] haplotype2;
   if (genotype)
      delete [] genotype;
   if (genotypeCounts)
      delete [] genotypeCounts;

   if (markers)
      delete [] markers;

   if (nAlleles)
      delete [] nAlleles;
   if (nGenotypes)
      delete [] nGenotypes;
  }

/**
 * ped is a 'nrow * ncol' matrix. Each row is a founder
 */
bool LDMax::SelectMarkers(int * new_markers, int *ped, int nrow, int ncol,
		int n_loci)
   {
   FreeWorkingArrays();
   NullWorkingArrays();

   CopyIntArray(&markers, new_markers, n_loci);
   nloci = n_loci;

   // Get no. of alleles (and genotypes) at each locus
   // as well as observed allele frequencies and genotype counts

   nAlleles = new int[nloci];
   nGenotypes = new int[nloci];
   haplotype1 = new int[nloci];
   haplotype2 = new int[nloci];
   genotype = new int[nloci];
   
   //alleleFrequencies = new (double *)[nloci];
   alleleFrequencies = (double **) R_alloc( (long) nloci, (int) sizeof(double *) );
   if(alleleFrequencies == NULL)
     error("Could not allocate enough memory for 'alleleFrequencies'!\n");
   

   for (int i = 0; i < nloci; i++)
      {
      int m = markers[i];

      // find out how many alleles there are at marker 'm'
      nAlleles[i] = CountAlleles(m, ped, nrow, ncol);
      //nAlleles[i] = ped.CountAlleles(m);
      nGenotypes[i] = nAlleles[i] * (nAlleles[i] + 1) / 2;
      
      NewVector(&alleleFrequencies[i], nAlleles[i]);
      }

   // calculate the total number of haplotypes and genotypes
   nhaplo = ngeno = 1;
   for (int i = 0; i < nloci; i++)
      {
      nhaplo *= nAlleles[i];
      ngeno *= nGenotypes[i];
      }

   if (nhaplo == 0)
      return false;

   NewVector(&haplotypeFrequencies, nhaplo, 1.0 / nhaplo);
   genotypeFrequencies = new double[ngeno];

   // count observed alleles and genotypes
   NewIntArray(&genotypeCounts, ngeno);
   count = 0;

   //for (int i = 0; i < ped.count; i++)
   for (int i = 0; i < nrow; i++)
      {
      //if (foundersOnly && !ped[i].isFounder()) continue;
      //if (!isGenotyped(ped[i])) continue;
      if (!isGenotyped(ped, ncol, i)) continue;

      //GetGenotype(ped[i]);
      GetGenotype(ped,ncol, i);
      genotypeCounts[GenotypeIndex()]++;
      count++;

      for (int j = 0; j < nloci; j++)
         {
         int m = markers[j];

         //alleleFrequencies[j][ped[i].markers[m].one-1]++;
         //alleleFrequencies[j][ped[i].markers[m].two-1]++;
	 /* ped has ncol2=2*numMarkers columns
	  * columns (1, 2) record two loci for marker 1
	  * columns (3, 4) record two loci for marker 2
	  * ....
	  */
	 // 'm' can be 0, 1, ...., numMarkers-1
         int ell1=ped[i*ncol+ (m*2)];
         int ell2=ped[i*ncol+ (m*2+1)];
         alleleFrequencies[j][ell1-1]++;
         alleleFrequencies[j][ell2-1]++;
         }
      }

   if (count < nhaplo)
      { return false; }

   for (int i = 0; i < nloci; i++)
      for (int j = 0; j < nAlleles[i]; j++)
         alleleFrequencies[i][j] /= (count * 2.0);

   // build genotype list
   //haplotypeList1 = new (int *)[ngeno];
   //haplotypeList2 = new (int *)[ngeno];
   haplotypeList1 = (int **) R_alloc( (long) ngeno, (int) sizeof(int *));
   if(haplotypeList1 == NULL)
     error("Could not allocate enough memory for 'haplotypeList1'!");
   haplotypeList2 = (int **) R_alloc( (long) ngeno, (int) sizeof(int *));
   if(haplotypeList2 == NULL)
     error("Could not allocate enough memory for 'haplotypeList2'!");

   for (int i = 0; i < ngeno; i++)
     {
     haplotypeList1[i] = NULL;
     haplotypeList2[i] = NULL;
     }

   NewIntArray(&lenHaploLists, ngeno);

   ResetHaplotype(haplotype1);
   do {
      CopyIntArray(&haplotype2, haplotype1, nloci);
      do {
         GetGenotype();
         int idx = GenotypeIndex();
         
         AppendHaploList(&(haplotypeList1[idx]), HaplotypeIndex(haplotype1), lenHaploLists[idx]);
         AppendHaploList(&(haplotypeList2[idx]), HaplotypeIndex(haplotype2), lenHaploLists[idx]);
         lenHaploLists[idx]++;
         } 
      while (IncrementHaplotype(haplotype2));
      } 
   while (IncrementHaplotype(haplotype1));

   UpdateGenotypeFrequencies();

   return true;
   }

int LDMax::CountAlleles(int marker, int *ped, int nrow, int ncol)
   {
   int result = 0;

   //for (int i=0; i < ped.count; i++)
   for (int i=0; i < nrow; i++)
      {
      int ell1=ped[i*ncol+ (marker*2)];
      int ell2=ped[i*ncol+ (marker*2+1)];
      if (ell2 > result)
         result = ell2;
      if (ell1 > result)
         result = ell1;
      //if (ped[i].markers[marker][1] > result)
         //result = ped[i].markers[marker][1];
      //if (ped[i].markers[marker][0] > result)
         //result = ped[i].markers[marker][0];
      }

   return result;
   }

void LDMax::LumpAlleles(int *ped, int nrow, int ncol, double threshold, 
		int marker)
   {
   // find out how many alleles there are
   int alleles = CountAlleles(marker, ped, nrow, ncol);

   if (alleles < 3) return;

   int * counts = new int [alleles+1];
   int * index = new int [alleles+1];

   // count all alleles ...
   for (int i = 0; i <= alleles; i++)
      {
      counts[i] = 0;
      index[i] = i;
      }

   //for (int i = 0; i < ped.count; i++)
   for (int i = 0; i < nrow; i++)
      {
      int ell1=ped[i*ncol+ (marker*2)];
      int ell2=ped[i*ncol+ (marker*2+1)];
      counts[ell1]++;
      counts[ell2]++;
      //counts[ped[i].markers[marker][0]]++;
      //counts[ped[i].markers[marker][1]]++;
      }

   // sort allele counts
   // Insertion sort should be OK, for N < 20
   for (int j = 2; j <= alleles; j++)
      {
      int tmp = counts[j];    // index[j] = j
      int i = j - 1;
      for ( ; (i > 0) && (counts[index[i]] < tmp); i--)
         index[i + 1] = index[i];
      index[i + 1] = j;
      }

   int total = 0;
   for (int i = 1; i <= alleles; i++)
      total += counts[i];
   int tresh = int(total * threshold);

   // recode alleles
   // all alleles where frequency < tresh are labelled N
   // use counts array to keep track of labels
   int N = 1;
   for (int i = 1; i <= alleles; i++)
      if (counts[index[i]] > tresh)
         {
         counts[index[i]] = i;
         N++;
         }
      else
         counts[index[i]] = N;
   counts[0] = 0;

   /***
    * It seems that 'current' only be defined and used in the following 
    * statements. So I (weiliang) commented it out. Nov. 6, 2005
   for (int i = 0; i < ped.count; i++)
      {
      int current = int[2];
      Alleles & current = ped[i].markers[marker];
      current[0] = counts[current[0]];
      current[1] = counts[current[1]];
      }
   ****/

   delete [] index;
   delete [] counts;
   }


void LDMax::AppendHaploList(int ** haplo_list, int value, int old_len)
   {
   int * new_list = new int[old_len + 1];
   for (int i = 0; i < old_len; i++)
      new_list[i] = (*haplo_list)[i];

   new_list[old_len] = value;

   if (*haplo_list)
      delete [] (*haplo_list);
   
   *haplo_list = new_list;
   }

bool LDMax::isGenotyped(int *ped, int ncol, int p)
   {
   // check if 'markers[i]' of subject 'p' contains missing allele (coded as 0)
   for (int i = 0; i < nloci; i++)
   {
      int m=markers[i];
      int ell1=ped[p*ncol+ (m*2)];
      int ell2=ped[p*ncol+ (m*2+1)];      
      //if (!p.markers[markers[i]].isKnown())
      if(ell1*ell2 == 0)
      { return false; }
   }
   return true;
   }

void LDMax::GetGenotype(int *ped, int ncol, int p)
   {
   for (int i = 0; i < nloci; i++)
      {
      int m = markers[i];
      int ell1=ped[p*ncol+ (m*2)];
      int ell2=ped[p*ncol+ (m*2+1)];      
      int a1 = mymax(ell1, ell2);
      int a2 = mymin(ell1, ell2);
      //int a1 = max(p.markers[m].one, p.markers[m].two);
      //int a2 = min(p.markers[m].one, p.markers[m].two);

      genotype[i] = a1 * (a1 - 1) / 2 + a2;
      }
   }

void LDMax::GetGenotype()
   {
   for (int i = 0; i < nloci; i++)
      {
      int a1 = mymax(haplotype1[i], haplotype2[i]);
      int a2 = mymin(haplotype1[i], haplotype2[i]);

      genotype[i] = a1 * (a1 - 1) / 2 + a2;
      }
   }

int LDMax::GenotypeIndex()
   {
   int result = 0;
   int factor = 1;

   for (int i = 0; i < nloci; i++)
      {
      result += (genotype[i] - 1) * factor;
      factor *= nGenotypes[i];
      }

   return result;
   }

void LDMax::ResetHaplotype(int * haplo)
   {
   for (int i = 0; i < nloci; i++)
      haplo[i] = 1;
   }

bool LDMax::isLastHaplotype(int * haplo)
   {
   for (int i = 0; i < nloci; i++)
      if (haplo[i] != nAlleles[i])
         return false;
   return true;
   }

bool LDMax::IncrementHaplotype(int * haplo)
   {
   int carry = 0;
   for (int i = 0; i < nloci; i++)
      if (haplo[i] != nAlleles[i])
         {
         haplo[i]++;
         while (carry--) haplo[--i] = 1;
         return true;
         }
      else
         carry++;

   return false;
   }

int LDMax::HaplotypeIndex(int * haplo)
   {
   int result = 0;
   int factor = 1;

   for (int i = 0; i < nloci; i++)
      {
      result += (haplo[i] - 1) * factor;
      factor *= nAlleles[i];
      }

   return result;
   }

void LDMax::TheWorks()
   {
   deltasq = delta = Dprime = D = LOD = 0.0;

   // ML stuff
   Maximize();
   CopyVector(&estimates, haplotypeFrequencies, nhaplo);
   logLKfull = LogLikelihood();

   // Null hypothesis
   NullFrequencies();
   CopyVector(&defaults, haplotypeFrequencies, nhaplo);
   logLKnull = LogLikelihood();

   if (nloci == 2 && nAlleles[0] == 2 && nAlleles[1] == 2)
      {
      double denominator =
            (estimates[0] + estimates[2]) * (estimates[1] + estimates[3]) *
            (estimates[0] + estimates[1]) * (estimates[2] + estimates[3]);

      if (denominator > 1e-7)
         {
         delta = (estimates[0] * estimates[3] - estimates[1] * estimates[2]);
         delta /= sqrt(denominator);
         deltasq = (estimates[0] * estimates[3] - estimates[1] * estimates[2]);
         deltasq *= deltasq;
         deltasq /= denominator;
         }
      }

   if (nloci == 2)
      {
      ResetHaplotype(haplotype1);
      do {
         double observed = estimates[HaplotypeIndex(haplotype1)];

         double p = alleleFrequencies[0][haplotype1[0]-1], q = 1 - p;
         double r = alleleFrequencies[1][haplotype1[1]-1], s = 1 - r;

         double expected = p * r;
         //double D = observed - expected, Dmax;
         double Dmax;
         D = observed - expected;

         if (fabs(D) > 1e-7)
            {
            Dmax = (D >= 0) ? mymind(p*s, q*r) : mymind(p*r, q*s);

            Dprime += fabs(D/(Dmax + TINY)) * expected;
            }
      } while (IncrementHaplotype(haplotype1));
      }

   ChiSq = 2 * (logLKfull - logLKnull);
   LOD = logLKfull/log(10) - logLKnull/log(10);
   dfFull = count - nhaplo + 1;
   dfNull = count;
   for (int i = 0; i < nloci; i++) dfNull -= nAlleles[i] - 1;
   pvalue = (dfFull < dfNull) ? chidist(mymaxd(ChiSq,0.0), dfNull - dfFull) : 1.0;
   }

double LDMax::NullFreq(int * haplo)
   {
   double result = 1.0;

   for (int i = 0; i < nloci; i++)
      result *= alleleFrequencies[i][haplo[i]-1];

   return result;
   }

void LDMax::NewIntArray(int ** target, int dim, int value)
   {
   *target = new int[dim];
   for (int i = 0; i < dim; i++)
      (*target)[i] = value;
   }

void LDMax::NewVector(double ** target, int dim, double value)
   {
   *target = new double[dim];
   for (int i = 0; i < dim; i++)
     (*target)[i] = value;
   }

void LDMax::CopyVector(double ** target, double * source, int dim)
   {
   if((*target))
   { delete[] *target; 
     *target = (double *)NULL;
   }
   *target = new double[dim];
 
   for (int i = 0; i < dim; i++)
     (*target)[i] = source[i];
   }

void LDMax::CopyIntArray(int ** target, int * source, int dim)
   {
   if((*target))
   { delete[] *target; 
     *target = (int *)NULL;
   }
   *target = new int[dim];
  
   for (int i = 0; i < dim; i++)
      (*target)[i] = source[i];
   }

void LDMax::GetHaplotype(int * haplo, int number)
   {
   int factor = 1;

   for (int i = 0; i < nloci; i++)
      factor *= nAlleles[i];

   for (int i = nloci - 1; i >= 0; i--)
      {
      factor /= nAlleles[i];
      haplo[i] = number / factor + 1;
      number = number % factor;
      }
   }

void LDMax::UpdateGenotypeFrequencies()
   {
   for (int i = 0; i < ngeno; i++)
     genotypeFrequencies[i] = TINY;

   for (int i = 0; i < ngeno; i++)

      for (int j = 0; j < lenHaploLists[i]; j++)
         {
         int h1 = haplotypeList1[i][j];
         int h2 = haplotypeList2[i][j];

         double f = haplotypeFrequencies[h1] * haplotypeFrequencies[h2];

         if (h1 == h2)
            genotypeFrequencies[i] += f;
         else
            genotypeFrequencies[i] += 2*f;
         }
   }

void LDMax::UpdateHaplotypeFrequencies()
   {
   double * next;
   NewVector(&next, nhaplo);

   for (int i = 0; i < ngeno; i++)
      {
      if (!genotypeCounts[i]) continue;
      double k = (double) genotypeCounts[i] / (double) count;

      for (int j = 0; j < lenHaploLists[i]; j++)
         {
         int h1 = haplotypeList1[i][j];
         int h2 = haplotypeList2[i][j];

         double f = haplotypeFrequencies[h1] * haplotypeFrequencies[h2];

         if (h1 != h2) f *= 2;

         next[h1] += k * f / genotypeFrequencies[i];
         next[h2] += k * f / genotypeFrequencies[i];
         }
      }

   for (int i = 0; i < nhaplo; i++)
      next[i] *= 0.5;

   CopyVector(&haplotypeFrequencies, next, nhaplo);
   delete[] next;
   }

void LDMax::NullFrequencies()
   {
   for (int i = 0; i < nhaplo; i++)
      {
      GetHaplotype(haplotype1, i);
      haplotypeFrequencies[i] = NullFreq(haplotype1);
      }
   UpdateGenotypeFrequencies();
   }

double LDMax::LogLikelihood()
   {
   double likelihood = 0;

   for (int i = 0; i < ngeno; i++)
      if (genotypeCounts[i])
         likelihood += log(genotypeFrequencies[i]) * genotypeCounts[i];

   return likelihood;
   }

void LDMax::Maximize()
   {
   double last = LogLikelihood(), current, delta;

   do {
      UpdateHaplotypeFrequencies();
      UpdateGenotypeFrequencies();
      current = LogLikelihood();
      delta = last - current;
      last = current;
   } while (fabs(delta) > fabs(last * TOL));
   }

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// The chi-squared distribution
//
double chidist(double x, double v)
   { return gammq (0.5 * v, 0.5 * x); }


double gammq ( double a, double x )
   {
   double gammser, gammcf, gln;

   if (x < 0.0 || a <= 0.0) error("Invalid arguments in routine gammq");
   if (x < (a + 1.0))
      {
      gser (&gammser, a, x, &gln);     // use the series representation
      return (1.0 - gammser);       // and take its complement
      }
   else
      {
      gcf ( &gammcf, a, x, &gln);      // use the continued fraction representation
      return gammcf;
      }
   }

void gser ( double * gamser, double a, double x, double * gln)
   {
   int   n;
   double sum, del, ap;

   *gln=gammln(a);
   if (x <= 0.0)
      {
      if (x < 0.0) error("x less than 0 in gamma series routine (gser)");
      *gamser = 0.0;
      return;
      }
   else
      {
      ap = a;
      del = sum = 1.0 / a;
      for (n = 1; n <= ITMAX; n++)
         {
         ++ ap;
         del *= x / ap;
         sum += del;
         if ( fabs(del) < fabs(sum) * EPS )
            {
            *gamser = sum * exp ( -x + a * log (x) - (*gln));
            return;
            }
         }
      error("a too large, ITMAX too small in gamma series routine (gser)");
      return;
      }
   }

void gcf ( double * gammcf, double a, double x, double * gln)
   {
   int i;
   double an, b, c, d, del, h;

   *gln = gammln(a);

   b = x + 1.0 - a;           // Setup for evaluating continued fraction by
   c = 1.0 / FPMIN;           // Lentz method (cf NRC 5.2)
   d = 1.0 / b;
   h = d;

   for ( i = 1; i <= ITMAX; i++ )   // Iterate to convergence
      {
      an = -i * ( i - a );
      b += 2.0;
      d = an * d + b;
      if ( fabs(d) < FPMIN ) d = FPMIN;
      c = b + an / c;
      if ( fabs(c) < FPMIN ) c = FPMIN;
      d = 1.0 / d;
      del = d * c;
      h *= del;
      if ( fabs(del-1.0) < EPS ) break;
      }
   if ( i > ITMAX ) error ("a too large, ITMAX too small in gamma countinued fraction (gcf)");
   *gammcf = exp (-x + a*log(x) - (*gln)) * h;
   }

// Gamma distribution functions
//

double gammln ( double xx )
   {
   double x, y, tmp, ser;
   static double cof[6] = { 76.18009172947146, -86.50532032941677, 24.01409824083091,
                      -1.231739572450155, 0.1208650973866179e-2, -0.5395239384953e-5 };
   int j;

   y = x = xx;

   tmp = x + 5.5;

   tmp -= ( x + 0.5 ) * log ( tmp );

   ser = 1.000000000190015;

   for ( j=0; j<=5; j++) ser += cof[j]/++y;

   return - tmp + log ( 2.5066282746310005 * ser/x);
   }

   
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

extern "C" {

/**
 * get pair-wise LD measures
 *  
 * founders -- a 'nrow*ncol' matrix containing founders' genotype information
 *             (not include sample info (i.e., family, pid, fid, mid, sex,
 *             affection))
 * nrow -- number of subjects
 * ncol -- 2*number of markers
 * numMarkers -- number of markers
 * pool --  all alleles where frequency < 'pool' are grouped together 
 * iprint -- indicates whether intermediate results should be print out
 * rMat -- 'numMarkers * numMarkers' matrix of pair-wise r (correlation)
 * r2Mat -- 'numMarkers * numMarkers' matrix of pair-wise r^2
 * DprimeMat -- 'numMarkers * numMarkers' matrix of pair-wise D' 
 * DMat -- 'numMarkers * numMarkers' matrix of pair-wise D 
 * chisqMat -- 'numMarkers * numMarkers' matrix of pair-wise log-likelihood ratio test statistics 
 * pvalMat -- 'numMarkers * numMarkers' matrix of pair-wise pvalues for log-likelihood ratio tests
 * nMat -- 'numMarkers * numMarkers' matrix of number of subjects
 * LODMat -- 'numMarkers * numMarkers' matrix of pair-wise LOD scores 
 **/
void getLD(int *founders, int *nrow, int *ncol,  int *numMarkers, double *pool, 
		int *iprint, double *rMat, double *r2Mat, double *DprimeMat, 
		double *DMat, double *chisqMat, double *pvalMat, int *nMat, double *LODMat)
{
   int i;
   int numMarkers2, nrow2, ncol2;
   double pool2 = *pool;
   bool   foundersOnly = 1;
   LDMax engine;
   int *markers, mysize=2;

   numMarkers2=*numMarkers;
   nrow2=*nrow; // number of subjects
   ncol2=*ncol; // 2*numMarkers2


   for(i=0; i<numMarkers2; i++)
   { engine.LumpAlleles(founders, nrow2, ncol2, pool2, i); }


   //markers.Dimension(2);
   markers=new int[mysize];
   engine.foundersOnly = foundersOnly;

   if((*iprint)==1) {
     printf("   M1   M2 al1 al2     N df(0) df(1)   ChiSq  pvalue delta  delta2    D'    D\n");
   }

   /********* calculate r^2 *************/
   for (markers[0] = 0; markers[0] < numMarkers2; markers[0]++)
   { 
     for (markers[1]=markers[0]+1; markers[1] < numMarkers2; markers[1]++)
       {
       if (!engine.SelectMarkers(markers, founders, nrow2, ncol2, mysize)) continue;
       engine.TheWorks();
       rMat[markers[1]*numMarkers2+markers[0]]=engine.delta;
       r2Mat[markers[1]*numMarkers2+markers[0]]=engine.deltasq;
       DprimeMat[markers[1]*numMarkers2+markers[0]]=engine.Dprime;
       DMat[markers[1]*numMarkers2+markers[0]]=engine.D;
       chisqMat[markers[1]*numMarkers2+markers[0]]=engine.ChiSq;
       pvalMat[markers[1]*numMarkers2+markers[0]]=engine.pvalue;
       nMat[markers[1]*numMarkers2+markers[0]]=nrow2;
       LODMat[markers[1]*numMarkers2+markers[0]]=engine.LOD;

       if((*iprint)==1) {
         printf("%5d%5d%4d%4d %5ld %5d %5d %7.2f %7.5f  %5.3f %5.3f %5.3f %5.3f %5.3f\n",
             markers[0]+1, markers[1]+1, engine.nAlleles[0], engine.nAlleles[1],
             engine.count, engine.dfNull, engine.dfFull, engine.ChiSq,
             engine.pvalue, engine.delta, engine.deltasq, engine.Dprime, 
	     engine.D, engine.LOD);

       }

     }
   }

  delete[] markers;
  return;

}

}

extern "C" {

/**
 * performing LD calculatations over sliding windows containing a
 * fixed *number* of markers.
 *
 * founders -- a 'nrow*ncol' matrix containing founders' genotype information
 *             (not include sample info (i.e., family, pid, fid, mid, sex,
 *             affection))
 * nrow -- number of subjects
 * ncol -- 2*number of markers
 * numMarkers -- number of markers
 * width -- size of window  
 * lag -- 'width/2' (i.e., half of the window)
 * pool --  all alleles where frequency < 'pool' are grouped together
 * iprint -- indicates whether intermediate results should be print out
 * rMat -- 'numMarkers * numMarkers' matrix of pair-wise r (correlation)
 * r2Mat -- 'numMarkers * numMarkers' matrix of pair-wise r^2
 * DprimeMat -- 'numMarkers * numMarkers' matrix of pair-wise D'
 * DMat -- 'numMarkers * numMarkers' matrix of pair-wise D 
 * chisqMat -- 'numMarkers * numMarkers' matrix of pair-wise log-likelihood ratio test statistics
 * pvalMat -- 'numMarkers * numMarkers' matrix of pair-wise pvalues for log-likelihood ratio tests
 * nMat -- 'numMarkers * numMarkers' matrix of number of subjects
 * LODMat -- 'numMarkers * numMarkers' matrix of pair-wise LOD scores
 **/   
void getLDband(int *founders, int *nrow, int *ncol,  int *numMarkers, 
		int *width, int *lag, double *pool, 
		int *iprint, double *rMat, double *r2Mat, double *DprimeMat, 
		double *DMat, double *chisqMat, double *pvalMat, int *nMat, double *LODMat)
{
   int i, j, li, lj;
   int numMarkers2, nrow2, ncol2, width2, lag2;
   double pool2 = *pool;
   bool   foundersOnly = 1;
   LDMax engine;
   int *markers, mysize=2;

   numMarkers2=*numMarkers;
   nrow2=*nrow; // number of subjects
   ncol2=*ncol; // 2*numMarkers2
   width2=*width;
   lag2=*lag;


   for(i=0; i<numMarkers2; i++)
   { engine.LumpAlleles(founders, nrow2, ncol2, pool2, i); }


   //markers.Dimension(2);
   markers=new int[mysize];
   engine.foundersOnly = foundersOnly;

   if((*iprint)==1) {
     printf("   M1   M2 al1 al2     N df(0) df(1)   ChiSq  pvalue delta  delta2    D'    D\n");
   }

   /********* calculate r^2 *************/
   for (markers[0] = 0; markers[0] < numMarkers2; markers[0]++)
   { for (markers[1]=markers[0]+1; markers[1] < numMarkers2; markers[1]++)
       {
       i=markers[0]+1;
       j=markers[1]+1;
       li=j-i+lag2-1;
       lj=i-j+lag2-1;

       if(!(li>0&&li<=width2) && !(lj>0&&lj<=width2))
       { continue; } // no calculation needed
       if (!engine.SelectMarkers(markers, founders, nrow2, ncol2, mysize)) continue;
       engine.TheWorks();
       if(li>0&&li<=width2)
       {
         rMat[(i-1)*width2+(li-1)]=engine.delta;
         r2Mat[(i-1)*width2+(li-1)]=engine.deltasq;
         DprimeMat[(i-1)*width2+(li-1)]=engine.Dprime;
         DprimeMat[(i-1)*width2+(li-1)]=engine.Dprime;
         DMat[(i-1)*width2+(li-1)]=engine.D;
         chisqMat[(i-1)*width2+(li-1)]=engine.ChiSq;
         pvalMat[(i-1)*width2+(li-1)]=engine.pvalue;
         nMat[(i-1)*width2+(li-1)]=nrow2;
         LODMat[(i-1)*width2+(li-1)]=engine.LOD;
       }

       if(lj>0&&lj<=width2)
       {
         rMat[(j-1)*width2+(lj-1)]=engine.delta;
         r2Mat[(j-1)*width2+(lj-1)]=engine.deltasq;
         DprimeMat[(j-1)*width2+(lj-1)]=engine.Dprime;
         DprimeMat[(j-1)*width2+(lj-1)]=engine.Dprime;
         DMat[(j-1)*width2+(lj-1)]=engine.D;
         chisqMat[(j-1)*width2+(lj-1)]=engine.ChiSq;
         pvalMat[(j-1)*width2+(lj-1)]=engine.pvalue;
         nMat[(j-1)*width2+(lj-1)]=numMarkers2;
         LODMat[(j-1)*width2+(lj-1)]=engine.LOD;
       }

       if((*iprint)==1) {
         printf("%5d%5d%4d%4d %5ld %5d %5d %7.2f %7.5f  %5.3f %5.3f %5.3f %5.3f %5.3f\n",
             markers[0]+1, markers[1]+1, engine.nAlleles[0], engine.nAlleles[1],
             engine.count, engine.dfNull, engine.dfFull, engine.ChiSq,
             engine.pvalue, engine.delta, engine.deltasq, engine.Dprime, 
	     engine.D, engine.LOD);

       }

     }
   }

  return;

}

}

