#include <vector>
#include <iterator>
#include <stdexcept>
#include <iostream>
#include <sstream>
#include <cstdio>
#include <stdlib.h>
#include <math.h>

#define UCEIL(x)   ((unsigned) ceil(x) )
#define UFLOOR(x)  ((unsigned) floor(x) )
#define MAXVAL(nbits,maxbits) ((unsigned) (nbits==maxbits)?(-1):((1 << nbits)-1))


using namespace std;


class bitArrayVector

{

public:

  ///////
  // Storage Type
  ///////
  //typedef unsigned int storage;

  ///////
  // Data members
  ///////

  // Maximum number of bits in a bitArrayVector element
  static const int maxbits = sizeof(unsigned)*8; 
  
  ///////
  // Constructors
  ///////

  // Base constructor
  bitArrayVector(unsigned nbits, unsigned len)
  {
    if (nbits > maxbits)
      throw logic_error("nbits > maxbits");
    if (nbits < 1)
      throw logic_error("nbits < 1");

    _size  = len;
    _nbits = nbits;
    _maxval = MAXVAL(nbits, maxbits);
    
    unsigned ncells = UCEIL( (len * nbits) / (double) maxbits);
    _data.resize( ncells );
  } 

  // Copy constructor
  bitArrayVector(const bitArrayVector& ba)
  {
    _size  = ba.size();
    _nbits = ba.nbits();
    _maxval = ba._maxval;
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );
    _data.assign( ba._data.begin(), ba._data.end() );
  } 

  bitArrayVector(const bitArrayVector* bap)
  {
    _size  = bap->size();
    _nbits = bap->nbits();
    _maxval = bap->_maxval;
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );
    _data.assign( bap->_data.begin(), bap->_data.end() );
  } 

  // Construct from existing C arrays 
  // packed=0, each array item corresponds to a single n-bit element
  // packed=1, array contains 'packed' data in the internal storage format of a bitArrayVector

  //bitArrayVector(bool[],       len, nbits, packed); 
  bitArrayVector(bool       data[], 
		 unsigned   len, 
		 unsigned   nbits, 
		 bool       packed = false)
  {
    int i;

    _size  = len;
    _nbits = nbits;
    _maxval = MAXVAL(nbits, maxbits);
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );

    // For bools, 'packed=true' and 'packed=false' are equivalent.
    for(i=0; i < len; i++)
      {
	setElement(i, (unsigned) data[i]);
      }
  }


  //bitArrayVector(short[],      len, nbits, packed); 
  bitArrayVector(short      data[], 
		 unsigned   len, 
		 unsigned   nbits, 
		 bool       packed = false)
  {
    int i;

    _size  = len;
    _nbits = nbits;
    _maxval = MAXVAL(nbits, maxbits);
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );

    if(packed)
      for(i=0; i < _data.size(); i++)
	{
	  setElement(i, (unsigned) data[i], sizeof(short), false);
	}
    else
      for(i=0; i < len; i++)
	{
	  setElement(i, (unsigned) data[i]);
	}
  }


  //bitArrayVector(int[],        length, nbits, packed);
  bitArrayVector(int        data[], 
		 unsigned   len, 
		 unsigned   nbits, 
		 bool       packed = false)
  {
    int i;

    _size  = len;
    _nbits = nbits;
    _maxval = MAXVAL(nbits, maxbits);
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );

    if(packed)
      for(i=0; i < _data.size(); i++)
	{
	  setElement(i, (unsigned) data[i], sizeof(int), false);
	}
    else
      for(i=0; i < len; i++)
	{
	  setElement(i, (unsigned) data[i]);
	}
  }



  bitArrayVector(unsigned   data[], 
		 unsigned   len, 
		 unsigned   nbits, 
		 bool       packed = false,
		 bool       verbose = false)
  {
    int i;

    cerr << "unsigned[]" << endl;

    _size  = len;
    _nbits = nbits;
    _maxval = MAXVAL(nbits, maxbits);
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );

    if(packed)
      for(i=0; i < _data.size(); i++)
	{
	  setElement(i, (unsigned) data[i], sizeof(unsigned)*8, true);
	}
    else
      for(i=0; i < len; i++)
	{
	  setElement(i, (unsigned) data[i], nbits, false);
	}
  }

  //bitArrayVector(double[],     len, nbits); // packed not permitted for doubles!
  bitArrayVector(double     data[], 
		 unsigned   len, 
		 unsigned   nbits
		 )

  {
    int i;

    _size  = len;
    _nbits = nbits;
    _maxval = MAXVAL(nbits, maxbits);
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );

    for(i=0; i < len; i++)
      {
	setElement(i, (unsigned) data[i]);
      }
  }

  //bitArrayVector(char[],       len, nbits, packed);
  bitArrayVector(char       data[], 
		 unsigned   len, 
		 unsigned   nbits, 
		 bool       packed = false)
  {
    int i;

    _size  = len;
    _nbits = nbits;
    _maxval = MAXVAL(nbits, maxbits);
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );

    if(packed)
      for(i=0; i < _data.size(); i++)
	{
	  setElement(i, (unsigned) data[i], sizeof(char), false);
	}
    else
      for(i=0; i < len; i++)
	{
	  setElement(i, (unsigned) data[i]);
	}
  }

  //bitArrayVector(char**,       len, nbits); 
  // Assumes data is an array of pointers to null terminated strings
  // that hold string representations of **integers**. If base=0, the
  // data can be provided in one of the standard representations:
  //     decimal:     0,    1,    2,    3, ..,   16, ..,  255
  //     octal  :    00,   01,   02,   03, ..,   20, .., 0377
  //     hex    :  0x00, 0x01, 0x02, 0x03, .., 0x10, .., 0xFF   
  // otherwise, the data is interpreted in the specified base. For
  // instance, base=2 permits
  //               0000, 0001, 0010, 0011, ..., 11000, ... , 11111111
 
  bitArrayVector(char**     data, 
		 unsigned   len, 
		 unsigned   nbits, 
		 bool       packed = false,
		 int        base=0)
  {
    int i;
    unsigned value;

    _size  = len;
    _nbits = nbits;
    _maxval = MAXVAL(nbits, maxbits);
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );

    value = (unsigned) strtoul( (const char *) data[i], NULL, base );

    if(packed)
      for(i=0; i < _data.size(); i++)
	{
	  setElement(i, (unsigned) value, sizeof(char), false);
	}
    else
      for(i=0; i < len; i++)
	{
	  setElement(i, (unsigned) value);
	}
  }

  //bitArrayVector(string,       len, nbits);
  // Assumes data is an array of strings that hold string
  // representations of **integers**. If base=0, the data can be
  // provided in one of the standard representations:
  //     decimal:     0,    1,    2,    3, ..,   16, ..,  255
  //     octal  :    00,   01,   02,   03, ..,   20, .., 0377
  //     hex    :  0x00, 0x01, 0x02, 0x03, .., 0x10, .., 0xFF   
  // otherwise, the data is interpreted in the specified base. For
  // instance, base=2 permits
  //               0000, 0001, 0010, 0011, ..., 11000, ... , 11111111
  bitArrayVector(string     data[], 
		 unsigned   len, 
		 unsigned   nbits, 
		 bool       packed = false,
		 int        base=0)
  {
    int i;
    unsigned value;

    _size  = len;
    _nbits = nbits;
    _maxval = MAXVAL(nbits, maxbits);
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );

    
    value = (unsigned) strtoul( (const char *) data[i].c_str(), NULL, base );

    if(packed)
      for(i=0; i < _data.size(); i++)
	{
	  setElement(i, (unsigned) value, sizeof(char), false);
	}
    else
      for(i=0; i < len; i++)
	{
	  setElement(i, (unsigned) value);
	}
  }

  // Construct from existing C++ vectors
  //bitArrayVector(vector<bool>,     nbits, packed);
  //bitArrayVector(vector<short>,    nbits, packed);
  //bitArrayVector(vector<int>,      nbits, packed);
  bitArrayVector(vector<unsigned> &data, 
		 unsigned   len,
		 unsigned   nbits, 
		 bool       packed = false)
  {
    int i;

    cerr << "vector<unsigned>" << endl;

    _size  = len;
    _nbits = nbits;
    _data.resize( UCEIL(_size * _nbits / (double) maxbits) );

    if(packed)
      for(i=0; i < data.size(); i++)
	{
	  cout << "packed data[" << i << "]=" << toHexString(data[i]) << endl;
	  _data[i] = data[i];
	}
    else
      for(i=0; i < data.size(); i++)
	{
	  cout << "element data[" << i << "]=" << toHexString(data[i]) << endl;
	  setElement(i, data[i]);
	}
  }
  //bitArrayVector(vector<double>,   nbits, packed);
  //bitArrayVector(vector<string>,   nbits, packed);

  
  //////////
  // Key operations: getElement() and setElement()
  //////////


  unsigned getElement(unsigned index, bool verbose=false)
  {
    if (index  > _size)
      throw logic_error("index out of range (index > size)");

    unsigned tmp, lower_byte, upper_byte, mask;


    unsigned startBit = (index  ) * _nbits;
    unsigned endBit   = (index+1) * _nbits - 1;
    unsigned byteIndexStart = UFLOOR((startBit) / maxbits );
    unsigned byteIndexEnd   = UFLOOR((endBit)   / maxbits );
    unsigned bitIndexStart  = startBit % maxbits;
    unsigned bitIndexEnd    = endBit   % maxbits;
    
    if(verbose)
      {
	cout << endl;
	cout << "index=" << index << endl;
	cout << "Bit indexes:" << startBit << ":" << endBit << endl;
	cout << "Real indexes: " << byteIndexStart << "[" << bitIndexStart << "]:" ;
	cout << byteIndexEnd <<   "[" << bitIndexEnd <<   "]" << endl;
	cout << endl;
      }
    
    if( byteIndexStart == byteIndexEnd ) // same byte
      {
	tmp = _data[byteIndexStart];

	// shift right to put bitIndexStart at bit 0
	tmp >>= bitIndexStart;

	// mask off all but the desired # of bits
	tmp &= _maxval;

	return tmp;
      }
    else if (byteIndexStart == byteIndexEnd -1 ) // adjacent bytes
      {

	if(verbose)
	  {
	    cout << endl;
	    cout << "Retrieving..." << endl;
	    cout << "index=" << index << endl;
	    cout << "Bit indexes:" << startBit << ":" << endBit << endl;
	    cout << "Real indexes: " << byteIndexStart << "[" << bitIndexStart << "]:" ;
	    cout << byteIndexEnd <<   "[" << bitIndexEnd <<   "]" << endl;
	    cout << endl;
	  }

	lower_byte = _data[byteIndexStart];
	upper_byte = _data[byteIndexEnd];

	if(verbose)
	  {
	    cout << "lower_byte: " << toHexString(lower_byte) << endl;
	    cout << "upper_byte: " << toHexString(upper_byte) << endl;
	    cout << endl;
	  }

	// step 1: LSBits are in upper bits of lower_byte 
	// shift these bits all the way right to position 0
	lower_byte >>= bitIndexStart-1;

	if(verbose)
	  {
	    cout << "lower_byte: " << toHexString(lower_byte) << endl;
	    cout << "upper_byte: " << toHexString(upper_byte) << endl;
	    cout << endl;
	  }
	    
	// step 2: MSBits are in the lower bits of upper_byte;
        // shif these left to make space for the LSBits
        upper_byte <<= _nbits - bitIndexEnd - 1;

	if(verbose)
	  {
	    cout << "lower_byte: " << toHexString(lower_byte) << endl;
	    cout << "upper_byte: " << toHexString(upper_byte) << endl;
	    cout << endl;
	    cout << "combined_byte: " << toHexString(upper_byte | lower_byte) << endl;
	    cout << endl;
	    cout << "combined_byte with mask: " << toHexString( (upper_byte | lower_byte) & _maxval ) << endl;
	    cout << endl;
	  }

	// step 4: combine & mask off extraneous bits
	return ( upper_byte | lower_byte ) & _maxval;
      }
    else
      throw logic_error("Unhandled case");
    
  }


  void setElement(unsigned index, unsigned value, bool verbose=false)
  {
    setElement(index, value, _nbits, verbose);
  }

  void setElement(unsigned index, unsigned value, unsigned nbits, bool verbose=false)
  {
    ostringstream msg;

    if(nbits==_nbits)
      {
	if (index  > _size)
	  {
	    msg << "index out of range (index=" << index << " > size=" << _size << ")";
	    throw logic_error( msg.str() );
	  }

	if (value > _maxval)
	  {
	    msg << "value (" << value << ") too large for number of bits (" << nbits << ") of bitArrayVector elements";
	    throw logic_error( msg.str() );
	  }
      }
    else
      {
	unsigned maxval=MAXVAL(nbits, maxbits);

	if(index >= _data.size())
	  {
	    msg << "index out of range (index=" << index << " > size=" << _data.size() << ")";
	    throw logic_error( msg.str() );
	  }
	if(value >= maxval )
	  {
	    msg << "setElement(" <<   index << "," <<  value << "," << nbits << "," << verbose << "): " << endl;
	    msg << "value (" << value << ") too large for specified number of bits (" << nbits << ")." << endl;
	    msg << "maximum value is " << maxval;
	    throw logic_error( msg.str() );
	  }
      }

    // cout << "  value=" << toHexString(value) << "  nbits=" << nbits
    // cout << "  _maxval=" << toHexString(_maxval) << endl;


    unsigned tmp, mask, upper_mask, upper_byte, lower_mask, lower_byte;

    unsigned startBit = (index  ) * nbits;
    unsigned endBit   = (index+1) * nbits - 1;
    unsigned byteIndexStart = UFLOOR((startBit) / maxbits );
    unsigned byteIndexEnd   = UFLOOR((endBit)   / maxbits );
    unsigned bitIndexStart  = startBit % maxbits;
    unsigned bitIndexEnd    = endBit   % maxbits;

//     cout << endl;
//     cout << "index=" << index << endl;
//     cout << "Bit indexes:" << startBit << ":" << endBit << endl;
//     cout << "Real indexes: " << byteIndexStart << "[" << bitIndexStart << "]:" ;
//     cout << byteIndexEnd <<   "[" << bitIndexEnd <<   "]" << endl;
//     cout << endl;
        


    // our basic mask
    mask = (1<<nbits) - 1;
        
    if( byteIndexStart == byteIndexEnd ) // same byte
      {
	// get the new data
	tmp = value;

	// shift left to put data and mask at bitIndexStart
	tmp  <<= bitIndexStart;
	mask <<= bitIndexStart;
	
	// clear bits in mask position
	_data[byteIndexStart] &= ~mask;

	// insert supplied bits
	_data[byteIndexStart] |= tmp;
      }
    else if (byteIndexStart == byteIndexEnd -1 ) // adjacent bytes
      {

	if(verbose)
	  {
	    cout << endl;
	    cout << "Storing..." << endl;
	    cout << "index=" << index << endl;
	    cout << "Bit indexes:" << startBit << ":" << endBit << endl;
	    cout << "Real indexes: " << byteIndexStart << "[" << bitIndexStart << "]:" ;
	    cout << byteIndexEnd <<   "[" << bitIndexEnd <<   "]" << endl;
	    cout << endl;

	    cout << "value:" << value << endl;
	    
	    cout << "value: " << toHexString(value) << endl;
	    cout << "mask:  " << toHexString(mask)  << endl;
	  }

	// move upper bits down to start of byte
	upper_mask  = mask  >> (nbits - endBit - 1);
	upper_byte  = value >> (nbits - endBit - 1);


	if(verbose)
	  {
	    cout << "upper_mask: " << toHexString(upper_mask) << endl;
	    cout << "upper_byte: " << toHexString(upper_byte) << endl;
	  }
	 
	// move lower bits up to end of byte
	lower_mask  = mask  << startBit-1;
	lower_byte  = value << startBit-1;

	if(verbose)
	  {
	    cout << "lower_mask: " << toHexString(lower_mask) << endl;
	    cout << "lower_byte: " << toHexString(lower_byte) << endl;

	    cout << "data[" << byteIndexStart << "]: " << getDataString(byteIndexStart) << endl;
	    cout << "data[" << byteIndexEnd << "]: " << getDataString(byteIndexEnd) << endl;
	  }

	// clear bits in mask position
	_data[byteIndexEnd]   &= ~ upper_mask;
	_data[byteIndexStart] &= ~ lower_mask;

	if(verbose)
	  {
	    cout << "data[" << byteIndexStart << "]: " << getDataString(byteIndexStart) << endl;
	    cout << "data[" << byteIndexEnd << "]: " << getDataString(byteIndexEnd) << endl;
	  }

	// insert supplied bits
	_data[byteIndexEnd]   |= upper_byte;
	_data[byteIndexStart] |= lower_byte;

	if(verbose)
	  {
	    cout << "data[" << byteIndexStart << "]: " << getDataString(byteIndexStart) << endl;
	    cout << "data[" << byteIndexEnd << "]: " << getDataString(byteIndexEnd) << endl;
	  }

      }
    else
      throw logic_error("Unhandled case");
    
  }


  ////////
  // Destructor
  ////////
  // ~bitArrayVector; // use default destructor


  ////////
  // Accessors
  ////////
  unsigned          size()   const { return _size;   }
  unsigned          nbits()  const { return _nbits;  }
  unsigned          maxval() const { return _maxval; }
  vector<unsigned>  data()   const { return _data;   }

  ////////
  // Data as C arrays
  ////////
  //bool[]      asBool();
  //short[]     asShort();
  //## int[]       asInt();
  //unsigned[]  asUnsigned();
  //double[]    asDouble();
  //char[]      asChar();
  //char**      asCharPtr();
  //string[]    asStrings();

  ////////
  // Data as packed C arrays
  ////////
  //bool[]      asPackedBool();
  //short[]     asPackedShort();
  //int[]       asPackedInt();
  unsigned*   asPackedUnsigned()
  {
    return &(_data.front()); 
  }
  //double[]    asPackedDouble();
  //char[]      asPackedChar();
  //char**      asPackedCharPtr();
  //string[]    asPackedStrings();


  ////////
  // Data as C++ vectors
  ////////
  //vector<bool>      asBoolVector();
  //vector<short>     asShortVector();
  //vector<int>       asIntVector();
  //## vector<unsigned>  asUnsignedVector();
  //vector<double>    asDoubleVector();
  //vector<char>      asCharVector();
  //vector<string>    asStringsVector();

  ////////
  // Data as packed C++ vectors
  ////////
  //bool[]      asPackedBool();
  //short[]     asPackedShort();
  //int[]       asPackedInt();
  vector<unsigned> *asPackedUnsignedVector()
  {
    return &_data;
  }
  //vector<double>    asPackedDoubleVector();
  //vector<char>      asPackedCharVector();
  //vector<string>    asPackedStringsVector();


 
  ////////
  // Vector indexing
  ////////
  //unsigned        operator[] (unsigned i) const; // j = bav[i];
  //bitArrayVector &operator[] (unsigned i);       // bav[i] = j;

  ////////
  // Subsetting
  ////////

  ////
  // Create/Assign new bitArrayVector from array of indexes
  ////
  //bitArrayVector  subset_index( unsigned index[], int len ) const;
  //bitArrayVector &subset_index( unsigned index[], int len );

  //bitArrayVector  subset_index( vector<unsigned> index ) const;
  //bitArrayVector &subset_index( vector<unsigned> index );

  ////
  // Create/Assign new bitArrayVector from array of indexes
  ////
  //bitArrayVector  subset_flag( vector<bool> index ) const;
  //bitArrayVector &subset_flag( vector<bool> index );

  //bitArrayVector  subset_flag( bool[] index ) const;
  //bitArrayVector &subset_flag( bool[] index );

  //bitArrayVector  subset_flag( int[] index ) const;
  //bitArrayVector &subset_flag( int[] index );

  ////
  // Create/Assign new bitArrayVector from (start,end) pairs
  ////
  //bitArrayVector  subset_range( unsigned start[], unsigned end[], int len ) const;
  //bitArrayVector &subset_range( unsigned start[], unsigned end[], int len );

  //bitArrayVector  subset_range( vector<unsigned> start, vector<unsigned> end ) const;
  //bitArrayVector &subset_range( vector<unsigned> start, vector<unsigned> end );



  ////
  // Extract array using indexes
  ////
  //bool[]      subset_index_asBool    ( unsigned index[], int len ) const;
  //bool[]      subset_flag_asBool     ( bool[] index ) const;
  //bool[]      subset_range_asBool    ( unsigned start[], unsigned end[], int len ) const;

  //short[]     subset_index_asShort   ( unsigned index[], int len ) const;
  //short[]     subset_flag_asShort    ( bool[] index ) const;
  //short[]     subset_range_asShort   ( unsigned start[], unsigned end[], int len ) const;

  /*
  int[]       subset_index_asInt     ( unsigned index[], int len ) const;
  int[]       subset_flag_asInt      ( bool[] index ) const;
  int[]       subset_range_asInt     ( unsigned start[], unsigned end[], int len ) const;
  */

  //unsigned[]  subset_index_asUnsigned( unsigned index[], int len ) const;
  //unsigned[]  subset_flag_asUnsigned ( bool[] index ) const;
  //unsigned[]  subset_range_asUnsigned( unsigned start[], unsigned end[], int len ) const;

  //double[]    subset_index_asDouble  ( double index[], int len ) const;
  //double[]    subset_flag_asDouble   ( bool[] index ) const;
  //double[]    subset_range_asDouble  ( double start[], double end[], int len ) const;

  //char[]      subset_index_asChar    ( unsigned index[], int len ) const;
  //char[]      subset_flag_asChar     ( char[] index ) const;
  //char[]      subset_range_asChar    ( unsigned start[], unsigned end[], int len ) const;

  //char**      subset_index_asCharPtr ( unsigned index[], int len ) const;
  //char**      subset_flag_asCharPtr  ( char[] index ) const;
  //char**      subset_range_asCharPtr ( unsigned start[], unsigned end[], int len ) const;


  ////////
  // Mathematical operators
  ////////
  
  // bitArrayVector operator+(bitArrayVector) const;
  // bitArrayVector operator-(bitArrayVector) const;
  // bitArrayVector operator*(bitArrayVector) const;
  // bitArrayVector operator/(bitArrayVector) const;
  // bitArrayVector operator|(bitArrayVector) const;
  // bitArrayVector operator&(bitArrayVector) const; 
  // bitArrayVector operator^(bitArrayVector) const; 

  // bitArrayVector operator+=(bitArrayVector) const;
  // bitArrayVector operator-=(bitArrayVector) const;
  // bitArrayVector operator*=(bitArrayVector) const;
  // bitArrayVector operator/=(bitArrayVector) const;
  // bitArrayVector operator|=(bitArrayVector) const;
  // bitArrayVector operator&=(bitArrayVector) const; 
  // bitArrayVector operator^=(bitArrayVector) const; 

  
  // byte[] byteSubset
  // intSubset
  // doubleSubset, charSubset(), charSubset(strings)
  // 
  // setAll, clearAll
  // bool[] testEqual(constant). 
  // bool[] testEqual(bitArrayVector)
  // tabulate() // All unique values
  // tabulate( patternVector )
  // 


  ///////
  // Utility 
  ///////

  // Convert uint to hex string
  static string toHexString(unsigned val)
  {
    int  buflen =  9;
    char buf[buflen];
    snprintf( buf, buflen, "%08x", val );
    buf[8] = 0;
    string s(buf);
    return s;
  }

  string getElementString(unsigned index)
  {
    return toHexString( getElement( index ) ) ;
  }

  string getDataString(unsigned index)
  {
    return toHexString( _data[ index ] ) ;
  }

  
private:

  // Note, by using unsigned, the maximum bitArrayVector length is 4GB
  // items

  // Number of elements
  unsigned _size;

  // Number of bits used for each element
  unsigned _nbits;

  // Largest storable value for nbits worth of data, can be used for
  // range checking, and is used internally as a bitmask.
  unsigned _maxval; 

  vector<unsigned> _data;
};



int main()
  {

    // test arrays with nbits raning from 1 to 17 bits, setting and
    // then getting each possible value five times to ensure proper
    // storage & retrieval, particularly across byte boundaries.
    // 17 was chosen as the largest value since half of the elements
    // will span adjacent storage cells, with the breakpoint cycling
    // over the bits.

    for(unsigned nbits=1; nbits <= 17; nbits++)
      {
	unsigned i, stored, retrieved;
	unsigned len = (1<<nbits)*5;

	cerr << "Create a " << nbits << "-bit bitArrayVector of length " << len << "..." << endl; 
	bitArrayVector bav(nbits, len);
	cerr << "Storage space used is: " << bav.data().size() << " bytes" << endl;

	cerr << "Filling from the front.." << endl;
	// set values
	for(i=0; i<len; i++)
	  bav.setElement(i, i % bav.maxval() );
	
	cerr << "Checking values.." << endl;
	// now check values
	for(i=0; i<len; i++)
	  {
 	    stored = i % bav.maxval();
	    bav.setElement(i, stored);
	    retrieved = bav.getElement(i);
	    
	    if(stored != retrieved)
	      {
		cout << "---------------------" << endl;
		cout << "Incorrect store/retrieve:  bav[" << i << "]:  Stored " << stored << " retrieved " << retrieved << endl;
		cout << "---------------------" << endl;
		bav.setElement(i, stored, true);
		retrieved = bav.getElement(i, true);
	      }
	  }


	cerr << "Filling from the back.." << endl;
	// set values
	for(i=len-1; i>-1; i--)
	  {
	    bav.setElement(i, i % bav.maxval() );
	    cerr << "i=" << i << endl;
	  }


	cerr << "Checking values.." << endl;
	// now check values
	for(i=len-1; i>-1; i--)
	  {
 	    stored = i % bav.maxval();
	    bav.setElement(i, stored);
	    retrieved = bav.getElement(i);
	    
	    if(stored != retrieved)
	      {
		cout << "---------------------" << endl;
		cout << "Incorrect store/retrieve:  bav[" << i << "]:  Stored " << stored << " retrieved " << retrieved << endl;
		cout << "---------------------" << endl;
		bav.setElement(i, stored, true);
		retrieved = bav.getElement(i, true);
	      }
	  }
	
	cerr << endl;
	cerr << endl;

      }

  /////
  // 2-bit bitArrayVector
  /////
  
  bitArrayVector bav2(2, 64/2);
  cout << "size:  " << bav2.size()        << endl;
  cout << "nbits: " << bav2.nbits()       << endl;
  cout << "#cells:" << bav2.data().size() << endl;
  
  for( unsigned i = 0; i < bav2.size(); i++ ) 
    {
      bav2.setElement(i, i % (1 << bav2.nbits()) );
    }
  
  cout << endl;
  cout << "bav2:" << endl;
  for( unsigned i = 0; i < bav2.size(); i++ ) 
    {
      cout << "[" << i << "]:" << bav2.getElement(i) << " ";
      if ((i+1)%8 ==0) cout << endl;
    }
  cout << endl;

  /////
  // 3-bit bitArrayVector
  /////

  bitArrayVector bav3 =  bitArrayVector(3, 64/3);

  for( unsigned i = 0; i < bav3.size(); i++ ) 
    {
      bav3.setElement(i, i % (1 << bav3.nbits()) );
    }

  cout << endl;
  cout << "bav3:" << endl;
  for( unsigned i = 0; i < bav3.size(); i++ ) 
    {
      cout << "[" << i << "]:" << bav3.getElement(i) << " ";
      if ((i+1)%8 ==0) cout << endl;
    }
  cout << endl;
  
  //////
  // Copy constructor
  //////
  
  bitArrayVector bav2_2(bav2);

  cout << endl;
  cout << "bav2_2:" << endl;
  for( unsigned i = 0; i < bav2_2.size(); i++ ) 
    {
      cout << "[" << i << "]:" << bav2_2.getElement(i) << " ";
      if ((i+1)%8 ==0) cout << endl;
    }
  cout << endl;
  
  ////////
  // Integer array constructor
  ////////
  unsigned nbits = 5;
  unsigned len = 40;
  unsigned ui[len];
  char tmp[3];

  for(unsigned i=0; i < len; i++)
    {
      ui[i] = i % (1<<nbits);

      sprintf(tmp, "%2d", i);

      cout << "ui[" << tmp  << "]=" << ui[i] << endl;
    }

  bitArrayVector bav5(ui, len, nbits, false);

  cout << endl;
  cout << "bav5:" << endl;
  for( unsigned i = 0; i < bav5.size(); i++ ) 
    {
      cout << "[" << i << "]:" << bav5.getElement(i) << " ";
      if ((i+1)%8 ==0) cout << endl;
    }
  cout << endl;

  ////////
  // Packed integer array constructor
  ////////

  cout << endl;
  cout << "bav5_p:" << endl;

  
  unsigned *packedUArray = bav5.asPackedUnsigned();

  for(int i=0; i<bav5.data().size(); i++ )
    {
      cout << bav5.toHexString(packedUArray[i]) << "-" ;
    }
  cout << endl;

  bitArrayVector bav5_p( packedUArray,
			 len, 
			 nbits, 
			 true);
  
  cout << "bav5_p:" << endl;

  cout << endl;
  cout << "bav5_p:" << endl;
  for( unsigned i = 0; i < bav5_p.size(); i++ ) 
    {
      cout << "[" << i << "]:" << bav5_p.getElementString(i) << " ";
      if ((i+1)%8 ==0) cout << endl;
    }
  cout << endl;

  ////////
  // Packed integer vector
  ////////
  
  vector<unsigned> *packedUVector = bav5.asPackedUnsignedVector();

  for(int i=0 ; i < packedUVector->size(); i++ )
    {
      cout << bav5.toHexString((*packedUVector)[i]) << "-" ;
    }
  cout << endl;

  bitArrayVector bav5_vp( *packedUVector,
			  len,
			  nbits,
			  true );

  cout << endl;
  cout << "bav5_vp:" << endl;
  for( unsigned i = 0; i < bav5_vp.size(); i++ ) 
    {
      cout << "[" << i << "]:" << bav5_vp.getElementString(i) << " ";
      if ((i+1)%8 ==0) cout << endl;
    }
  cout << endl;

}
