/*****
 Dr. Albert Vernon Smith  kindly provided the original 
 version of this function. 
 Albert also made a great contribution to improve 
 the files LdMax.cpp, LdMax.h, getLD.R.


Albert Vernon Smith, Ph.D.
Cold Spring Harbor Laboratory
HapMap Data Coordination Center
E-mail: smithav@cshl.edu

*****/

#include "LdMax.h"

extern "C" {

/**
 * performing LD calculatations over sliding windows containing markers
 * within a specified distance.
 *
 * founders -- a 'nrow*ncol' matrix containing founders' genotype information
 *             (not include sample info (i.e., family, pid, fid, mid, sex,
 *             affection))
 * nrow -- number of subjects
 * ncol -- 2*number of markers
 * numMarkers -- number of markers 
 * pos -- positions of markers
 * maxDist -- size of the window based on distance
 * pool --  all alleles where frequency < 'pool' are grouped together
 * iprint -- indicates whether intermediate results should be print out
 * rMat -- 'numMarkers * numMarkers' matrix of pair-wise r (correlation)
 * r2Mat -- 'numMarkers * numMarkers' matrix of pair-wise r^2
 * DprimeMat -- 'numMarkers * numMarkers' matrix of pair-wise D'
 * DMat -- 'numMarkers * numMarkers' matrix of pair-wise D
 * chisqMat -- 'numMarkers * numMarkers' matrix of pair-wise log-likelihood ratio test statistics
 * pvalMat -- 'numMarkers * numMarkers' matrix of pair-wise pvalues for log-likelihood ratio tests
 * nMat -- 'numMarkers * numMarkers' matrix of number of subjects 
 * LODMat -- 'numMarkers * numMarkers' matrix of pair-wise LOD scores
 **/   
void getLDdist(int *founders, int *nrow, int *ncol,  int *numMarkers, int *pos,
                int *maxDist, double *pool, int *iprint, double *rMat, 
                double *r2Mat, double *DprimeMat, double *DMat, 
                double *chisqMat, double *pvalMat, int *nMat, double *LODMat)
{
   int i;

   int numMarkers2, nrow2, ncol2, maxDist2;
   double pool2 = *pool;
   bool   foundersOnly = 1;
   LDMax engine;
   int *markers, mysize=2;

   numMarkers2=*numMarkers;
   nrow2=*nrow; // number of subjects
   ncol2=*ncol; // 2*numMarkers2
   maxDist2 = *maxDist;

   for(i=0; i<numMarkers2; i++)
   { engine.LumpAlleles(founders, nrow2, ncol2, pool2, i); }


   //markers.Dimension(2);
   markers=new int[mysize];
   engine.foundersOnly = foundersOnly;

   if((*iprint)==1) {
     printf("   M1   M2 al1 al2     N df(0) df(1)   ChiSq  pvalue delta  delta2    D'    D     LOD\n");

   }

   /********* calculate r^2 *************/
   for (markers[0] = 0; markers[0] < numMarkers2; markers[0]++)
   {
     for (markers[1]=markers[0]+1; markers[1] < numMarkers2; markers
[1]++)
     {
       if ( abs(pos[markers[0]] - pos[markers[1]]) > maxDist2 )
         continue;

       if (!engine.SelectMarkers(markers, founders, nrow2, ncol2, mysize)) 
         continue;

       engine.TheWorks();

       rMat[markers[1]*numMarkers2+markers[0]]=engine.delta;
       r2Mat[markers[1]*numMarkers2+markers[0]]=engine.deltasq;
       DprimeMat[markers[1]*numMarkers2+markers[0]]=engine.Dprime;
       DMat[markers[1]*numMarkers2+markers[0]]=engine.D;
       chisqMat[markers[1]*numMarkers2+markers[0]]=engine.ChiSq;
       pvalMat[markers[1]*numMarkers2+markers[0]]=engine.pvalue;
       nMat[markers[1]*numMarkers2+markers[0]]=nrow2;
       LODMat[markers[1]*numMarkers2+markers[0]]=engine.LOD;

       if((*iprint)==1) {
         printf("%5d%5d%4d%4d %5ld %5d %5d %7.2f %7.5f  %5.3f %5.3f %5.3f %5.3f %5.2f\n",
             markers[0]+1, markers[1]+1, engine.nAlleles[0],
engine.nAlleles[1],
             engine.count, engine.dfNull, engine.dfFull, engine.ChiSq,
             engine.pvalue, engine.delta, engine.deltasq,
engine.Dprime, engine.D, engine.LOD);
       }
     }
   }

   delete [] markers ;

  return;

}

}

